/*
* Jalview - A Sequence Alignment Editor and Viewer
* Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/
package MCview;

import jalview.datamodel.*;

import jalview.schemes.ResidueProperties;

import java.awt.*;

import java.util.*;
import jalview.analysis.AlignSeq;


public class PDBChain {
    public String id;
    public Vector bonds = new Vector();
    public Vector atoms = new Vector();
    public Vector residues = new Vector();
    public int offset;
    public Sequence sequence;
    public boolean isVisible = true;
    public int pdbstart = 0;
    public int pdbend = 0;
    public int seqstart = 0;
    public int seqend = 0;

    public PDBChain(String id) {
        this.id = id;
    }

    public String print() {
        String tmp = "";

        for (int i = 0; i < bonds.size(); i++) {
            tmp = tmp + ((Bond) bonds.elementAt(i)).at1.resName + " " +
                ((Bond) bonds.elementAt(i)).at1.resNumber + " " + offset +
                "\n";
        }

        return tmp;
    }

    void makeExactMapping(AlignSeq as, Sequence s1)
    {
        int pdbpos =   as.getSeq2Start()-2;
        int alignpos = s1.getStart() + as.getSeq1Start()-3;

        for(int i=0; i<as.astr1.length(); i++)
        {
            if (as.astr1.charAt(i) != '-')
            {
              alignpos++;
            }

            if (as.astr2.charAt(i) != '-')
            {
              pdbpos++;
            }

            if (as.astr1.charAt(i) == as.astr2.charAt(i))
            {
                Residue res = (Residue) residues.elementAt(pdbpos);
                Enumeration en = res.atoms.elements();
                while (en.hasMoreElements())
                {
                  Atom atom = (Atom) en.nextElement();
                  atom.alignmentMapping = alignpos;
                }
            }
        }

    }


    public void makeCaBondList()
    {
        for (int i = 0; i < (residues.size() - 1); i++)
        {
            Residue tmpres = (Residue) residues.elementAt(i);
            Residue tmpres2 = (Residue) residues.elementAt(i + 1);
            Atom at1 = tmpres.findAtom("CA");
            Atom at2 = tmpres2.findAtom("CA");

            if ((at1 != null) && (at2 != null))
            {
                if (at1.chain.equals(at2.chain))
                {
                    makeBond(at1, at2);
                }
            }
            else
              System.out.println("not found "+i);
        }
    }

    public void makeBond(Atom at1, Atom at2) {
        float[] start = new float[3];
        float[] end = new float[3];

        start[0] = at1.x;
        start[1] = at1.y;
        start[2] = at1.z;

        end[0] = at2.x;
        end[1] = at2.y;
        end[2] = at2.z;

        bonds.addElement(new Bond(start, end, at1, at2));
    }

    public void makeResidueList() {
        int count = 0;
        StringBuffer seq = new StringBuffer();

        int i, iSize = atoms.size()-1;
        for (i = 0; i < iSize; i++)
        {
            Atom tmp = (Atom) atoms.elementAt(i);
            int resNumber = tmp.resNumber;
            int res = resNumber;

            if (i == 0) {
                offset = resNumber;
            }

            Vector resAtoms = new Vector();

            resAtoms.addElement((Atom) atoms.elementAt(i));
            i++;
            resNumber = ((Atom) atoms.elementAt(i)).resNumber;

            //Add atoms to a vector while the residue number
            //remains the same
            while ((resNumber == res) && (i < atoms.size())) {
                resAtoms.addElement((Atom) atoms.elementAt(i));
                i++;

                if (i < atoms.size()) {
                    resNumber = ((Atom) atoms.elementAt(i)).resNumber;
                } else {
                    resNumber++;
                }
            }

            //We need this to keep in step with the outer for i = loop
            i--;

            //Make a new Residue object with the new atoms vector
            residues.addElement(new Residue(resAtoms, resNumber - 1, count));
            count++;

            Residue tmpres = (Residue) residues.lastElement();
            Atom tmpat = (Atom) tmpres.atoms.elementAt(0);

            // Keep totting up the sequence
            if (ResidueProperties.getAA3Hash().get(tmpat.resName) == null)
            {
                seq.append("X") ;
               //  System.err.println("PDBReader:Null aa3Hash for " +
               //     tmpat.resName);
            } else {

                seq.append(ResidueProperties.aa[((Integer) ResidueProperties.getAA3Hash()
                                                                                  .get(tmpat.resName)).intValue()]);
            }
        }

        if(id.length()<1 || id.equals(" "))
           id = "_";

        sequence = new Sequence(id, seq.toString(), 1, seq.length());
      //  System.out.println("PDB Sequence is :\nSequence = " + seq);
     //   System.out.println("No of residues = " + residues.size());
    }

    public void setChargeColours() {
        for (int i = 0; i < bonds.size(); i++) {
            try {
                Bond b = (Bond) bonds.elementAt(i);

                if (b.at1.resName.equalsIgnoreCase("ASP") ||
                        b.at1.resName.equalsIgnoreCase("GLU")) {
                    b.startCol = Color.red;
                } else if (b.at1.resName.equalsIgnoreCase("LYS") ||
                        b.at1.resName.equalsIgnoreCase("ARG")) {
                    b.startCol = Color.blue;
                } else if (b.at1.resName.equalsIgnoreCase("CYS")) {
                    b.startCol = Color.yellow;
                } else {
                    //int atno = ((Integer) ResidueProperties.getAA3Hash().get(b.at1.resName.toUpperCase())).intValue();
                    b.startCol = Color.lightGray;
                }

                if (b.at2.resName.equalsIgnoreCase("ASP") ||
                        b.at2.resName.equalsIgnoreCase("GLU")) {
                    b.endCol = Color.red;
                } else if (b.at2.resName.equalsIgnoreCase("LYS") ||
                        b.at2.resName.equalsIgnoreCase("ARG")) {
                    b.endCol = Color.blue;
                } else if (b.at2.resName.equalsIgnoreCase("CYS")) {
                    b.endCol = Color.yellow;
                } else {
                    //int atno = ((Integer) ResidueProperties.getAA3Hash().get(b.at2.resName.toUpperCase())).intValue();
                    b.endCol = Color.lightGray;
                }
            } catch (Exception e) {
                Bond b = (Bond) bonds.elementAt(i);
                b.startCol = Color.gray;
                b.endCol = Color.gray;
            }
        }
    }


    public void setChainColours(jalview.schemes.ColourSchemeI cs)
    {
        Bond b;
        for (int i = 0; i < bonds.size(); i++) {
            try {
              b = (Bond) bonds.elementAt(i);

              ( (Bond) bonds.elementAt(i)).startCol = cs.findColour(
                  ResidueProperties.aa[ ( (Integer) ResidueProperties.aa3Hash.
                                         get(b.at1.resName)).intValue()]
                  );

              b.endCol = cs.findColour(
                  ResidueProperties.aa[ ( (Integer) ResidueProperties.aa3Hash.
                                         get(b.at2.resName)).intValue()]
                  );

            } catch (Exception e)
            {
                b = (Bond) bonds.elementAt(i);
                b.startCol = Color.gray;
                b.endCol = Color.gray;
            }
        }
    }



    public void setChainColours(Color col)
    {
        for (int i = 0; i < bonds.size(); i++)
        {
            Bond tmp = (Bond) bonds.elementAt(i);
            tmp.startCol = col;
            tmp.endCol = col;
        }
    }
}
