    /*
    * Jalview - A Sequence Alignment Editor and Viewer
    * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
    *
    * This program is free software; you can redistribute it and/or
    * modify it under the terms of the GNU General Public License
    * as published by the Free Software Foundation; either version 2
    * of the License, or (at your option) any later version.
    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
    */
    package jalview.io;

    import jalview.datamodel.*;

    import java.util.Vector;


    /**
     * DOCUMENT ME!
     *
     * @author $author$
     * @version $Revision: 1.4 $
     */
    public class AppletFormatAdapter
    {
        /** DOCUMENT ME!! */
        public static final Vector formats = new Vector();

        public static String INVALID_CHARACTERS = "Contains invalid characters";

        public static String SUPPORTED_FORMATS = "Formats currently supported are\n" +
                                                 "Fasta, MSF, Clustal, BLC, PIR, MSP, and PFAM";

        static
        {
            formats.addElement("BLC");
            formats.addElement("CLUSTAL");
            formats.addElement("FASTA");
            formats.addElement("MSF");
            formats.addElement("PileUp");
            formats.addElement("PIR");
            formats.addElement("PFAM");
        }


        public static String FILE = "File";
        public static String URL = "URL";
        public static String PASTE = "Paste";
        public static String CLASSLOADER = "ClassLoader";

        AlignFile afile = null;

        /**
         * DOCUMENT ME!
         *
         * @param inFile DOCUMENT ME!
         * @param type DOCUMENT ME!
         * @param format DOCUMENT ME!
         *
         * @return DOCUMENT ME!
         */
        public SequenceI[] readFile(String inFile, String type, String format)
            throws java.io.IOException
        {
            try
            {
                if (format.equals("FASTA"))
                {
                    afile = new FastaFile(inFile, type);
                }
                else if (format.equals("MSF"))
                {
                    afile = new MSFfile(inFile, type);
                }
                else if (format.equals("PileUp"))
                {
                    afile = new PileUpfile(inFile, type);
                }
                else if (format.equals("CLUSTAL"))
                {
                    afile = new ClustalFile(inFile, type);
                }
                else if (format.equals("BLC"))
                {
                    afile = new BLCFile(inFile, type);
                }
                else if (format.equals("PIR"))
                {
                    afile = new PIRFile(inFile, type);
                }
                else if (format.equals("PFAM"))
                {
                    afile = new PfamFile(inFile, type);
                }


                return afile.getSeqsAsArray();
            }
            catch (Exception e)
            {
              System.err.println("Failed to read alignment using the '" + format +
                                 "' reader.\n"+e);

              if(e.getMessage()!=null && e.getMessage().startsWith(INVALID_CHARACTERS))
                throw new java.io.IOException(e.getMessage());

              // Finally test if the user has pasted just the sequence, no id
              if(type.equalsIgnoreCase("Paste"))
              {
                try{
                  // Possible sequence is just residues with no label
                  afile = new FastaFile(">UNKNOWN\n" + inFile, "Paste");
                  return afile.getSeqsAsArray();
                }
                catch(Exception ex)
                {
                  if(ex.toString().startsWith(INVALID_CHARACTERS))
                    throw new java.io.IOException(e.getMessage());

                  ex.printStackTrace();
                }
              }

              // If we get to this stage, the format was not supported
              throw new java.io.IOException(SUPPORTED_FORMATS);
            }
        }


        /**
         * DOCUMENT ME!
         *
         * @param format DOCUMENT ME!
         * @param seqs DOCUMENT ME!
         *
         * @return DOCUMENT ME!
         */
        public String formatSequences(String format,
                                      Vector seqs,
                                      boolean jvsuffix)
        {
            SequenceI[] s = new SequenceI[seqs.size()];

            for (int i = 0; i < seqs.size(); i++)
                s[i] = (SequenceI) seqs.elementAt(i);

            try
            {
                AlignFile afile = null;

                if (format.equalsIgnoreCase("FASTA"))
                {
                    afile = new FastaFile();
                }
                else if (format.equalsIgnoreCase("MSF"))
                {
                    afile = new MSFfile();
                }
                else if (format.equalsIgnoreCase("PileUp"))
                {
                    afile = new PileUpfile();
                }
                else if (format.equalsIgnoreCase("CLUSTAL"))
                {
                    afile = new ClustalFile();
                }
                else if (format.equalsIgnoreCase("BLC"))
                {
                    afile = new BLCFile();
                }
                else if (format.equalsIgnoreCase("PIR"))
                {
                    afile = new PIRFile();
                }
                else if (format.equalsIgnoreCase("PFAM"))
                {
                    afile = new PfamFile();
                }

                afile.addJVSuffix(jvsuffix);

                afile.setSeqs(s);

                return afile.print();
            }
            catch (Exception e)
            {
                System.err.println("Failed to write alignment as a '" + format +
                    "' file\n");
                e.printStackTrace();
            }

            return null;
        }
    }
